package nl.tbdev.asterisk.demo;
import java.io.BufferedReader;
import java.io.OutputStreamWriter;
import java.net.Socket;

/**
 * @author kersing
 *
 * Detector defines a single presence detector with associated calling information
 */
public class Detector {
	private String channel;
	private String watchChannel;
	private String extension;
	private String context;
	private String timeout;
	private String callerid;
	private String ibutton;
	private boolean busy;
	private Socket socket;
	private BufferedReader reader;
	private OutputStreamWriter writer;
	
	public Detector() {
		channel = "";
		extension = "";
		context = "";
		timeout = "";
		callerid = "";
		ibutton = "";
		watchChannel = "";
		busy = false;
		socket = null;
		writer = null;
	}
	
	public Detector(String ch, String ext, String co, String to, String cid, String wCh, String ib) {
		channel = ch;
		extension = ext;
		context = co;
		timeout = to;
		callerid = cid;
		ibutton = ib;
		watchChannel = wCh;
		busy = false;
		socket = null;
		writer = null;
	}
	
	/**
	 * Returns the callerid.
	 * @return String
	 */
	public String getCallerid() {
		return callerid;
	}

	/**
	 * Returns the channel.
	 * @return String
	 */
	public String getChannel() {
		return channel;
	}

	/**
	 * Returns the context.
	 * @return String
	 */
	public String getContext() {
		return context;
	}

	/**
	 * Returns the extension.
	 * @return String
	 */
	public String getExtension() {
		return extension;
	}

	/**
	 * Returns the ibutton.
	 * @return String
	 */
	public String getIbutton() {
		return ibutton;
	}

	/**
	 * Returns the timeout.
	 * @return String
	 */
	public String getTimeout() {
		return timeout;
	}

	/**
	 * Returns the watchChannel.
	 * @return String
	 */
	public String getWatchChannel() {
		return watchChannel;
	}

	/**
	 * Returns the busy.
	 * @return boolean
	 */
	public boolean isBusy() {
		return busy;
	}

	/**
	 * Sets the busy.
	 * @param busy The busy to set
	 */
	public void setBusy(boolean busy) {
		this.busy = busy;
	}

	/**
	 * Returns the socket.
	 * @return Socket
	 */
	public Socket getSocket() {
		return socket;
	}

	/**
	 * Sets the socket.
	 * @param socket The socket to set
	 */
	public void setSocket(Socket socket) {
		this.socket = socket;
		try {
			this.writer = new OutputStreamWriter(socket.getOutputStream());
		} catch (Exception e) {
			try {
				socket.close();
			} catch (Exception ex) {}
			this.socket = null;
		}
	}

	/**
	 * Returns the writer.
	 * @return OutputStreamWriter
	 */
	public OutputStreamWriter getWriter() {
		return writer;
	}

	/**
	 * Method setReader.
	 * @param in
	 */
	public void setReader(BufferedReader in) {
		reader = in;
	}

}
