package nl.tbdev.asterisk.demo;

import com.dalsemi.onewire.OneWireAccessProvider;
import com.dalsemi.onewire.adapter.DSPortAdapter;
import com.dalsemi.onewire.container.OneWireContainer;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.Enumeration;
import java.util.Vector;

public class Presence extends Thread  {
	private static Socket asterisk;
	private static OutputStreamWriter out;
	private static BufferedReader in;
	private static Vector detectors;
	private static Server server = null;
	private static String aServer = null;

	Presence() {
		asteriskConnect();
		server = new Server(detectors);
	}	

	private void asteriskConnect() {
		String input;
		try {		
			// open socket to asterisk
			asterisk = new Socket(aServer,5038);
			out = new OutputStreamWriter(asterisk.getOutputStream());
			in = new BufferedReader(new InputStreamReader(asterisk.getInputStream()));
	
			// login
			input = in.readLine();
			out.write("Action: Login\r\n");
			// Enter you own credentials here
			out.write("Username: username\r\n");
			out.write("Secret: password\r\n\r\n");
			out.flush();
			boolean success = false;
			while (!success) {
				input = in.readLine();
				if (input.startsWith("Response: Succes")) {
					success = true;
				}
			}
		} catch (Exception e) {
			System.out.println("Exception occured while connecting to asterisk: "+e);
			try {
				out.close();
			} catch (Exception ex) {}
			try {
				in.close();
			} catch (Exception ex) {}
			try {
				asterisk.close();
			} catch (Exception ex) {}
		}	
	}

	public void run() {
		String input;
		Detector detector = null;
		Vector v = null;
		while (true) {
			try {
				while (true) {
					// re-open connection if closed
					if (in == null) {
						asteriskConnect();
					}
					
					// read up till the next empty line
					v = new Vector();
					input = in.readLine();
					input = input.trim();
					while ( ! "".equals(input)) {
						// show user what we got
						System.out.println(input);

						// and add to storage
						v.addElement(input);
						
						// read next line
						input = in.readLine();
						input = input.trim();
					}
					System.out.println("Command complete");

					String command = getValue(v, "Event:");
					
					// did we get a Hangup Event?
					if (command.indexOf("Hangup") >= 0) {
						// find channel and signal we're done
						for (Enumeration d_enum = detectors.elements(); d_enum.hasMoreElements(); ) {
							Detector d = (Detector) d_enum.nextElement();
							
							if (input.indexOf(d.getWatchChannel()) < 0 ) {
								d.setBusy(false);
								showImage(d, "", "", "");
							}
						}
					}
					
					// was it a new extension?
					if (command.indexOf("Newexten") >= 0) {
						// find if we're watching this channel
						for (Enumeration d_enum = detectors.elements(); d_enum.hasMoreElements(); ) {
							Detector d = (Detector) d_enum.nextElement();
							
							if (input.indexOf(d.getWatchChannel()) < 0 ) {
								// yep, this one is of interest
								String newContext = getValue(v, "Context:");
								String newExten = getValue(v, "Extension:");
								String newPri = getValue(v, "Priority:");
								showImage(d, newContext, newExten, newPri);
							}
						}
					}
				}
			} catch (Exception e) {
				System.out.println("Exception on Asterisk connection: "+e);
			}
		}
	}

	/**
	 * Method getValue. Return value of parameter specified by string from values stored in vector.
	 * @param v
	 * @param string
	 * @return String
	 */
	private String getValue(Vector v, String string) {
		int max = v.size();
		String val;
		for (int i = 0; i < max; i++) {
			val = (String) v.elementAt(i);
			if (val.startsWith(string)) {
				int pos = val.indexOf(": ");
				return val.substring(pos+2);
			}
		}
		return null;
	}

	/**
	 * Method showImage.
	 * @param socket
	 */
	private void showImage(Detector d, String context, String exten, String pri) {
		if (d.getSocket() == null ) {
			return;
		}

		if ("1".equals(pri) || "".equals(pri)) {
			String image = "idle.gif";
			if ("1".equals(pri)) {
				int val = 0;
				try {
					val = Integer.parseInt(exten) - 700;
				} catch (Exception e) {}
				image = "slide"+val+".gif";
			}
			
			try {
				System.out.println(" >>>> display: "+image);
				d.getWriter().write(image+"\n");
				d.getWriter().flush();
			} catch (Exception e) {
				System.out.println("Exception: "+e);
			}
		}
	}


	public static void main(String args[])  {
		// get server name from command line
		if (args.length != 1) {
			System.out.println("Missing server address, please add to commandline");
			System.exit(1);
		}
		aServer = args[0];

		// add devices to the list
		detectors = new Vector();
		Detector d;
//		Detector d = new Detector("SIP/prive1@prive1", "701", "switch", "45000", "424", "SIP/172.16.1.200:5060", "E8000006E184AD01");
//		detectors.add(d);

		// create and start communications with asterisk
		Presence p = new Presence();
		p.start();

		try {

			// Connect to one-wire and start probing
			DSPortAdapter adapter = OneWireAccessProvider.getDefaultAdapter();
			System.out.println();
			System.out.println("Adapter: " + adapter.getAdapterName()
				      + " Port: " + adapter.getPortName());
		       	System.out.println();

			adapter.beginExclusive(true);
			adapter.setSearchAllDevices();
			adapter.setSpeed(DSPortAdapter.SPEED_REGULAR);
			adapter.targetAllFamilies();

			boolean forever = true;
			while ( forever) {
				try {
					OneWireContainer owd;
					String address;
					for (Enumeration owd_enum = adapter.getAllDeviceContainers(); owd_enum.hasMoreElements(); ) {
						owd = ( OneWireContainer ) owd_enum.nextElement();

						// check if it's the one we're after
						address = owd.getAddressAsString();
						System.out.println("Found: "+address);
						for (Enumeration d_enum = detectors.elements(); d_enum.hasMoreElements(); ) {
							d = (Detector) d_enum.nextElement();
							
							if ( d.getIbutton().equals(address) ) {
								activate(d);
							}
						}
						// wait a second before next try
						try {
							Thread.sleep(1000);
						} catch (Exception e) {}
					}
				} catch (Exception e) {
					// System.out.println(e);
				}
			}
			adapter.endExclusive();
			adapter.freePort();
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	static void activate(Detector d) {
		System.out.println("Initiate call");
		try {
			if ( !d.isBusy() ) {
				d.setBusy(true);
				out.write("Action: Originate\r\n");
				out.write("Channel: "+d.getChannel()+"\r\n");
				out.write("Exten: "+d.getExtension()+"\r\n");
				out.write("Context: "+d.getContext()+"\r\n");
				out.write("Priority: 1\r\n");
				out.write("Timeout: "+d.getTimeout()+"\r\n");
				out.write("CallerID: "+d.getCallerid()+"\r\n\r\n");
				out.flush();
			}
		} catch (Exception e) {}
	}
}
// vi: ai ts=4 sw=4
